function f = BMIE_TI(signal,lev)

% BMIE_TI:      Implements a Bayesian method to estimate the intensity of a Poisson signal 
%               based on a translation invariant Multiscale Multiplicative Innovations Model.       
% Usage
%               f = BMIE_TI(signal,lev)
% Inputs
%   signal	    1-d Noisy signal, length(signal)= 2^J.
%   lev         Coarsest resolution level; lev << log2(length(signal)).
% Outputs
%   f		       Estimate of the underlying intensity function.
% References
%               Timmermann, K.E. & Nowak, R.D. (1999). Multiscale modeling and estimation
%               of Poisson processes with applications to photon-limited imaging.
%               "IEEE Trans. Inf. Theor.", 45, 846-862.

%Initialisations
n=length(signal);
J=log2(n);

%Filter Generation: unnormalized Haar filters
qmf=MakeONFilter('Haar',2)*sqrt(2.);

%Definition of shape parameters and weights
s1=1;
s2=100;
s3=10000;
p1=0.001;
p2=0.07;
p3=1-p1-p2;

%Inizialization
fSHsum=zeros(1,n);
fSH=zeros(n,1);
delta=zeros(n,1);
teta=zeros(n,1);
lambda=zeros(2*n,1);
signalSH=signal;
for sh=1:2^(J-lev)
   
%Shift
signalSH=rshift(signalSH);
    
%Wavelet and scaling coefficients generation
[wcoef,scoef]=fwst_po(signalSH,lev,qmf);
lambda(1:2^lev)=scoef(1:2^lev);

%Estimation
    for j=lev:J-1
        indjk=[2^j-2^lev+1:2^(j+1)-2^lev];
        indjk1=[2^(j+1)-2^lev+1:2:2^(j+2)-2^lev-1];
        indjk2=indjk1+1;
        
        b1=2*s1+scoef(indjk);
        b2=2*s2+scoef(indjk);
        b3=2*s3+scoef(indjk);
        
%Evaluation of delta(j,k)
a1=exp(gammaln(s1+scoef(indjk1))+gammaln(s1+scoef(indjk2))-gammaln(2*s1+scoef(indjk))-2*gammaln(s1)+gammaln(2*s1));
a1=p1*a1.*b2.*b3;
a2=exp(gammaln(s2+scoef(indjk1))+gammaln(s2+scoef(indjk2))-gammaln(2*s2+scoef(indjk))-2*gammaln(s2)+gammaln(2*s2));
a2=p2*a2.*b1.*b3;
a3=exp(gammaln(s3+scoef(indjk1))+gammaln(s3+scoef(indjk2))-gammaln(2*s3+scoef(indjk))-2*gammaln(s3)+gammaln(2*s3));
a3=p3*a3.*b1.*b2;
num=wcoef(indjk).*(a1+a2+a3);
den=a1.*b1+a2.*b2+a3.*b3;

%It can happen that a1==0, a2==0 and a3==0 and this implies that num==0 and den==0, but num tends towards 0 
%faster than den, so num/den==0. This is the reason why we find the indexes of the elements that are zero 
%in den (also in num) and define den(index)==1. Infact num(index)==0 and num(index)/den(index)=num(index)/1.0=0.

index=find(den==0);
den(index)=1.;
delta(indjk)=num./den;

%Evaluation of teta(j,k)         
teta(indjk)=lambda(indjk).*delta(indjk);

%Refinement        
lambda(indjk1)=0.5*(lambda(indjk)+teta(indjk));
lambda(indjk2)=0.5*(lambda(indjk)-teta(indjk));
    end
    
fSH=lambda(n-2^lev+1:2*n-2^lev,1)';
fSHsum=fSHsum+lkshift(fSH,sh);
fSH(:)=0;
delta(:)=0;
teta(:)=0;
lambda(:)=0;
end

%Denoised estimate of the original signal using the translation invariant 
%Multiscale Multiplicative Innovations Model.
f=(fSHsum/(2^(J-lev)));

% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom.
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus.
%
% mailto: T.Sapatinas@ucy.ac.cy 
