function [est,ppp,qqq] = BMSMShrink(sig, pqind)

%BMSMShrink:  Implements a Bayesian method to estimate the intensity of a Poisson signal 
%             based on a translation invariant Multiscale Model.
% Usage 
%             [est,ppp,qqq] = BMSMShrink(sig, pqind)
% Inputs 
%    sig      Input vector of counts  (assumed length a power of 2).
%    pqind    Indicator for default values of parameters
%		        p and q (i.e., 1) or estimation through method
%		        of marginal maximum likelihood (i.e., 2).
% Outputs
%    est      Estimate of the underlying intensity function.
%    ppp      (Uncorrected) vector of mixing probabilities.
%    qqq      (Uncorrected) vector of beta shape parameters.
% References
%             Kolaczyk, E.D. (1999). Bayesian multiScale models for Poisson processes. 
%             "Journal of the American Statistical Association", 94, 920-933. 

%Initialisations
n = length(sig);
J = log2(n);

%Create decomposition table of signal, using pairwise sums, keeping just the values that 
%are *not* redundant under the shift-invariant scheme.  This is very similar to TI-tables
%in Donoho and Coifman's TI-denoising framework.
dmat = zeros(J+1,n);
dmat(1,:) = sig;
for D=0:1:J-1,
  nD = 2^(J-D); nDo2 = nD/2;
  for l=0:1:2^D-1,
    ind = [l*nD+1:(l+1)*nD];
    x = dmat(D+1,ind);
    lsumx = x(1:2:nD-1) + x(2:2:nD);
    rx = rshift(x);
    rsumx = rx(1:2:nD-1) + rx(2:2:nD);
    dmat(D+2,ind) = [lsumx rsumx];
  end
end

% Set values of (hyper)parameters.
if (pqind == 1)
  ppp = 0.900*ones(J,1);  
  qqq = 1.00*ones(J,1);  
elseif (pqind == 2)
  qqq = ones(J,1);
  ppp = MLpEstTI(dmat,qqq);
else,
  'pqind must be 1 or 2. Hit ctrl-c to abort.'
  return
end

% Test that p and q are within acceptable boundaries.
% If not, set to boundary values.
for j=1:1:J,
  if ( ppp(j) >= 1.0 )
    pp(j) = 1.0;  qq(j) = 1.0;
  elseif ( 0.0 < ppp(j) & ppp(j) < 1.0 & qqq(j) <= 0.0 )
    pp(j) = ppp(j);  qq(j) = 1.0;
  elseif ( 0.0 < ppp(j) & ppp(j) < 1.0 & qqq(j) > 0.0 )
    pp(j) = ppp(j);  qq(j) = qqq(j);
  elseif ( ppp(j) < 0.0 )
    pp(j) = 0.0;  qq(j) = 1.0;
  else,
    'Observed an unanticipated value of p and/or q.'
    ppp(j)
    qqq(j)
    pp(j) = 0.90; qq(j) = 1.0;
    return
  end
end

% Beginning with the total number of counts, working from coarse
% scales (i.e., deep depths) upwards, gradually build up the MSPB
% shrinkage estimate by multiplying by appropriate shrinkage factors
% at each level.
est = dmat(J+1,:);

for D = J:-1:1,
%plot(est)
%pause
nD = 2^(J-D+1); nDo2 = nD/2;
for l=0:1:2^(D-1)-1,
% Set indexing so as to pick off blocks of size 2^(J-D+1)
% when shrinking estimates at depth D+1 down to finer
% scale at depth D.
    ind = [l*nD+1:(l+1)*nD];
    xxD = dmat(D,ind);
    estvec = est(ind);
    
% In the first half of the vector of D+1-depth estimates,
% we can shrink using the D-depth counts in the order
% in which they appear.
    estl = estvec(1:nDo2);
    xxDl = xxD(1:2:nD-1);  xxDr = xxD(2:2:nD);
    ss = sfunc(pp(D),qq(D),xxDl,xxDr);
    nestl = reshape([estl.*ss ; estl.*(1 - ss)], [1,nD]);
    
% In the second half of the vector of D+1-depth counts,
% we right-shift the D-depth counts, compute the shrunken
% values, and left-shift these values back to the order
% of the above.
    estr = estvec(nDo2+1:nD);
    sxxD = rshift(xxD);
    xxDl = sxxD(1:2:nD-1);  xxDr = sxxD(2:2:nD);
    ss = sfunc(pp(D),qq(D),xxDl,xxDr);
    nestr = reshape([estr.*ss ; estr.*(1 - ss)], [1,nD]);
    nestr = lshift(nestr);
    
% Combine the estimates from both halves of the D+1-depth
% counts, and store.
    est(ind) = 0.5.*( nestl + nestr );
  end
end 

% Copyright (c) 1997.
%
% Eric Kolaczyk
% Department of Mathematics and Statistics
% Boston University
% 111 Cummington Avenue
% Boston, MA 02215
% USA
%
% mailto: kolaczyk@math.bu.edu 
%
% Revised: 01/09/05
