function f = Simulations(test,n,M,scale,ordind,lev,hint,show)

% Simulations:     The main routine to produce the figures and tables given in
%                  the paper by Besbeas, De Feis & Sapatinas (2002). We refer to this paper 
%                  which discusses in detail wavelet shrinkage methods for Poisson counts.
%                  Various estimators have been compared in an extensive simulation 
%                  on a variety of sample sizes, test functions, count settings,
%                  primary resolution levels and wavelet filters. Various different
%                  criteria have been used to measure performance.
% Usage
%                  f = Simulations(test,n,M,scale,ordind,lev,hint,show)
% Inputs
%   test           "Integer pointing" to Signal function:
%			          'Smooth', 'Clipped Blocks', 'Bumps', 'HeaviSine', 'Angles', 'Spikes', 'Bursts'.
%   n              Sample size (i.e. number of data points) -- power of 2.
%   M              Total number of simulations per setting.
%  scale           Scaling factor for constructing low, medium and high count settings.
%  ordind          Order indicator dictating whether skewness alone is corrected for, 
%                  or both skewness and kurtosis are corrected for.
%                  (Enter 3 if you want skewness only; 
%                  Enter 4 if you want both skweness and kyrtosis).
%   lev            Coarsest resolution level; lev << log2(n).
%   hint 	       Quadrature mirror filter for wavelet transform
%		             (Enter 1 if you want Symmlet 8; Enter 2 if you want Coiflet 3).
%   show	          Enter 1 if you want figures showing boxplots of the various criterias to measure. 
% Outputs      
%                  A structure f with the following fields:
%   amse_f         Average Mean Squared Error.
%   rmse_f	       Root Mean Squared Error.
%   rmsb_f         Root Mean Squared Bias.
%   amxdv_f	       Average Maximum Deviation.
%   alinorm_f	    Average L1-Norm.
%   acputime_f     Average CPU Time.
% References
%                  Besbeas, P., De Feis, I. & Sapatinas, T. (2004). A comparative
%                  simulation study of wavelet shrinkage estimators for Poisson counts.
%                  International Statistical Review, Vol. 72, 209-237.
% Notes    
%                  Wavelet methods used : 
%                  Anscombe Transformation: 1)  hard minimax threshold
%                                           2)  soft minimax threshold
%                                           3)  hard universal threshold
%                                           4)  soft universal threshold
%                                           5)  hard cross validation threshold
%                                           6)  soft cross validation threshold
%                                           7)  translation invariant hard minimax threshold
%                                           8)  translation invariant soft minimax threshold
%                                           9)  translation invariant hard universal threshold
%                                           10) translation invariant soft universal threshold
%                                           11) translation invariant hard cross validation threshold
%                                           12) translation invariant soft cross validation threshold
%                  Fisz Transformation:     13) hard minimax threshold
%                                           14) soft minimax threshold
%                                           15) hard universal threshold
%                                           16) soft universal threshold
%                                           17) hard cross validation threshold
%                                           18) soft cross validation threshold
%                  Haar Thesholds:          19) translation invariant hard threshold
%                                           20) translation invariant soft threshold
%                  Corrected Thresholds:    21) hard threshold
%                                           22) soft threshold
%                                           23) translation invariant hard threshold
%                                           24) translation invariant soft threshold
%                  25) Modulation Estimator
%                  26) Translation Invariant Bayesian Multiscale Method  
%                  27) Translation Invariant Bayesian Multiscale Multiplicative Innovations Model 
%                  28) l1-Penalised Likelihood Method 

if nargin < 8
	show=1;
end
if nargin < 7
	hint=1;
end
if nargin < 6
	lev=4;
end
if nargin < 5
   ordind=3;
end
if nargin < 4
	scale=20;
end
if nargin < 3
	M=20;
end
if nargin < 2
	n=256;
end

%String argument
names = {'Smooth' 'Clipped Blocks' 'Bumps' 'Angles' 'Spikes' 'Bursts'};
wavelet= {'Symmlet 8' 'Coiflet 3'};
%
if hint==1,
   h=MakeONFilter('Symmlet',8);
else
   h=MakeONFilter('Coiflet',3);
end
%
fprintf('\n test=%3i n=%4i M=%3i scale=%3i ordind=%1i lev=%1i h=%1i show=%3i\n',...
        test,n,M,scale,ordind,lev,hint,show);
fname = sprintf('t%in%iM%is%io%il%ih%is%i.out',test,n,M,scale,ordind,lev,hint,show); %filename containing the output
fidname = fopen(fname,'w');
fprintf(fidname,'\n test function = %5i \n', test);
fprintf(fidname,'\n sample size = %5i \n', n);
fprintf(fidname,'\n number of simulations = %5i \n', M);
fprintf(fidname,'\n scale = %5i \n', scale);
fprintf(fidname,'\n lev = %5i \n', lev);
fprintf(fidname,'\n ordind = %5i \n', ordind);
fprintf(fidname,'\n quadrature mirror filter = %5i \n', hint);
fprintf(fidname,'\n\n');

%initialize the simulations
% MSE
mse_anscombeMin_H = zeros(M,1);
mse_anscombeMin_S = zeros(M,1);
mse_anscombeUni_H = zeros(M,1);
mse_anscombeUni_S = zeros(M,1);
mse_anscombeCV_H = zeros(M,1);
mse_anscombeCV_S = zeros(M,1);

mse_anscombeMin_TI_H = zeros(M,1);
mse_anscombeMin_TI_S = zeros(M,1);
mse_anscombeUni_TI_H = zeros(M,1);
mse_anscombeUni_TI_S = zeros(M,1);
mse_anscombeCV_TI_H = zeros(M,1);
mse_anscombeCV_TI_S = zeros(M,1);

mse_fiszMin_H = zeros(M,1);
mse_fiszMin_S = zeros(M,1);
mse_fiszUni_H = zeros(M,1);
mse_fiszUni_S = zeros(M,1);
mse_fiszCV_H = zeros(M,1);
mse_fiszCV_S = zeros(M,1);

mse_haar_TI_H = zeros(M,1);
mse_haar_TI_S = zeros(M,1);

mse_corrected_H = zeros(M,1);
mse_corrected_S = zeros(M,1);

mse_corrected_TI_H = zeros(M,1);
mse_corrected_TI_S = zeros(M,1);

mse_modulation = zeros(M,1);

mse_BMSMShrink_TI = zeros(M,1);

mse_MMIM_TI = zeros(M,1);

mse_PENl1 = zeros(M,1);

% MXDV
mxdv_anscombeMin_H = zeros(M,1);
mxdv_anscombeMin_S = zeros(M,1);
mxdv_anscombeUni_H = zeros(M,1);
mxdv_anscombeUni_S = zeros(M,1);
mxdv_anscombeCV_H = zeros(M,1);
mxdv_anscombeCV_S = zeros(M,1);

mxdv_anscombeMin_TI_H = zeros(M,1);
mxdv_anscombeMin_TI_S = zeros(M,1);
mxdv_anscombeUni_TI_H = zeros(M,1);
mxdv_anscombeUni_TI_S = zeros(M,1);
mxdv_anscombeCV_TI_H = zeros(M,1);
mxdv_anscombeCV_TI_S = zeros(M,1);

mxdv_fiszMin_H = zeros(M,1);
mxdv_fiszMin_S = zeros(M,1);
mxdv_fiszUni_H = zeros(M,1);
mxdv_fiszUni_S = zeros(M,1);
mxdv_fiszCV_H = zeros(M,1);
mxdv_fiszCV_S = zeros(M,1);

mxdv_haar_TI_H = zeros(M, 1);
mxdv_haar_TI_S = zeros(M, 1);

mxdv_corrected_H = zeros(M, 1);
mxdv_corrected_S = zeros(M, 1);

mxdv_corrected_TI_H = zeros(M, 1);
mxdv_corrected_TI_S = zeros(M, 1);

mxdv_modulation = zeros(M, 1);

mxdv_BMSMShrink_TI = zeros(M,1);

mxdv_MMIM_TI = zeros(M,1);

mxdv_PENl1 = zeros(M,1);

% L1 Norm
l1_anscombeMin_H = zeros(M,1);
l1_anscombeMin_S = zeros(M,1);
l1_anscombeUni_H = zeros(M,1);
l1_anscombeUni_S = zeros(M,1);
l1_anscombeCV_H = zeros(M,1);
l1_anscombeCV_S = zeros(M,1);

l1_anscombeMin_TI_H = zeros(M,1);
l1_anscombeMin_TI_S = zeros(M,1);
l1_anscombeUni_TI_H = zeros(M,1);
l1_anscombeUni_TI_S = zeros(M,1);
l1_anscombeCV_TI_H = zeros(M,1);
l1_anscombeCV_TI_S = zeros(M,1);

l1_fiszMin_H = zeros(M,1);
l1_fiszMin_S = zeros(M,1);
l1_fiszUni_H = zeros(M,1);
l1_fiszUni_S = zeros(M,1);
l1_fiszCV_H = zeros(M,1);
l1_fiszCV_S = zeros(M,1);

l1_haar_TI_H = zeros(M,1);
l1_haar_TI_S = zeros(M,1);

l1_corrected_H = zeros(M,1);
l1_corrected_S = zeros(M,1);

l1_corrected_TI_H = zeros(M,1);
l1_corrected_TI_S = zeros(M,1);

l1_modulation = zeros(M,1);

l1_BMSMShrink_TI = zeros(M,1);

l1_MMIM_TI = zeros(M,1);

l1_PENl1 = zeros(M,1);

% CPU Time
cpu_anscombeMin_H = zeros(M,1);
cpu_anscombeMin_S = zeros(M,1);
cpu_anscombeUni_H = zeros(M,1);
cpu_anscombeUni_S = zeros(M,1);
cpu_anscombeCV_H = zeros(M,1);
cpu_anscombeCV_S = zeros(M,1);

cpu_anscombeMin_TI_H = zeros(M,1);
cpu_anscombeMin_TI_S = zeros(M,1);
cpu_anscombeUni_TI_H = zeros(M,1);
cpu_anscombeUni_TI_S = zeros(M,1);
cpu_anscombeCV_TI_H = zeros(M,1);
cpu_anscombeCV_TI_S = zeros(M,1);

cpu_fiszMin_H = zeros(M,1);
cpu_fiszMin_S = zeros(M,1);
cpu_fiszUni_H = zeros(M,1);
cpu_fiszUni_S = zeros(M,1);
cpu_fiszCV_H = zeros(M,1);
cpu_fiszCV_S = zeros(M,1);

cpu_haar_TI_H = zeros(M,1);
cpu_haar_TI_S = zeros(M,1);

cpu_corrected_H = zeros(M,1);
cpu_corrected_S = zeros(M,1);

cpu_corrected_TI_H = zeros(M,1);
cpu_corrected_TI_S = zeros(M,1);

cpu_modulation = zeros(M,1);

cpu_BMSMShrink_TI = zeros(M,1);

cpu_MMIM_TI = zeros(M,1);

cpu_PENl1 = zeros(M,1);

%Initialising the estimates
anscombeMin_H = zeros(1,n);
anscombeMin_S = zeros(1,n);
anscombeUni_H = zeros(1,n);
anscombeUni_S = zeros(1,n);
anscombeCV_H = zeros(1,n);
anscombeCV_S = zeros(1,n);

anscombeMin_TI_H = zeros(1,n);
anscombeMin_TI_S = zeros(1,n);
anscombeUni_TI_H = zeros(1,n);
anscombeUni_TI_S = zeros(1,n);
anscombeCV_TI_H = zeros(1,n);
anscombeCV_TI_S = zeros(1,n);

fiszMin_H = zeros(1,n);
fiszMin_S = zeros(1,n);
fiszUni_H = zeros(1,n);
fiszUni_S = zeros(1,n);
fiszCV_H = zeros(1,n);
fiszCV_S = zeros(1,n);

haar_TI_H = zeros(1,n);
haar_TI_S = zeros(1,n);

corrected_H = zeros(1,n);
corrected_S = zeros(1,n);

corrected_TI_H = zeros(1,n);
corrected_TI_S = zeros(1,n);

modulation = zeros(1,n);

BMSMShrink_TI = zeros(1,n);

MMIM_TI = zeros(1,n);

PENl1 = zeros(1,n);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
all_anscombeMin_H = zeros(1,n);
all_anscombeMin_S = zeros(1,n);
all_anscombeUni_H = zeros(1,n);
all_anscombeUni_S = zeros(1,n);
all_anscombeCV_H = zeros(1,n);
all_anscombeCV_S = zeros(1,n);

all_anscombeMin_TI_H = zeros(1,n);
all_anscombeMin_TI_S = zeros(1,n);
all_anscombeUni_TI_H = zeros(1,n);
all_anscombeUni_TI_S = zeros(1,n);
all_anscombeCV_TI_H = zeros(1,n);
all_anscombeCV_TI_S = zeros(1,n);

all_fiszMin_H = zeros(1,n);
all_fiszMin_S = zeros(1,n);
all_fiszUni_H = zeros(1,n);
all_fiszUni_S = zeros(1,n);
all_fiszCV_H = zeros(1,n);
all_fiszCV_S = zeros(1,n);

all_haar_TI_H = zeros(1,n);
all_haar_TI_S = zeros(1,n);

all_corrected_H = zeros(1,n);
all_corrected_S = zeros(1,n);

all_corrected_TI_H = zeros(1,n);
all_corrected_TI_S = zeros(1,n);

all_modulation = zeros(1,n);

all_BMSMShrink_TI = zeros(1,n);

all_MMIM_TI = zeros(1,n);

all_PENl1 = zeros(1,n);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
all_anscombeMin_H2 = zeros(1,n);
all_anscombeMin_S2 = zeros(1,n);
all_anscombeUni_H2 = zeros(1,n);
all_anscombeUni_S2 = zeros(1,n);
all_anscombeCV_H2 = zeros(1,n);
all_anscombeCV_S2 = zeros(1,n);

all_anscombeMin_TI_H2 = zeros(1,n);
all_anscombeMin_TI_S2 = zeros(1,n);
all_anscombeUni_TI_H2 = zeros(1,n);
all_anscombeUni_TI_S2 = zeros(1,n);
all_anscombeCV_TI_H2 = zeros(1,n);
all_anscombeCV_TI_S2 = zeros(1,n);

all_fiszMin_H2 = zeros(1,n);
all_fiszMin_S2 = zeros(1,n);
all_fiszUni_H2 = zeros(1,n);
all_fiszUni_S2 = zeros(1,n);
all_fiszCV_H2 = zeros(1,n);
all_fiszCV_S2 = zeros(1,n);

all_haar_TI_H2 = zeros(1,n);
all_haar_TI_S2 = zeros(1,n);

all_corrected_H2 = zeros(1,n);
all_corrected_S2 = zeros(1,n);

all_corrected_TI_H2 = zeros(1,n);
all_corrected_TI_S2 = zeros(1,n);

all_modulation_2 = zeros(1,n);

all_BMSMShrink_TI_2 = zeros(1,n);

all_MMIM_TI_2 = zeros(1,n);

all_PENl1_2 = zeros(1,n);

%generate the intensity function
signal = MakeIntensities(names(test),n);

%produce the different count settings
signal = scale * (1 + signal);

%Starting the Simulations
nsignal=zeros(1,n);
for m=1:M
    fprintf(' Evaluating Simulation Number: ');
    
%generate the data
    nsignal(1,:) = MakeNoisyIntensities(names(test),n,scale);
    
%calculate the signal smooth estimates
    t=cputime;
    anscombeMin_H = recanscombe(nsignal(1,:),'MinMax','Hard',lev,h);
    cpu_anscombeMin_H(m,1) = cputime -t;

    t=cputime;
    anscombeMin_S = recanscombe(nsignal(1,:),'MinMax','Soft',lev,h);
    cpu_anscombeMin_S(m,1) = cputime -t;

    t=cputime;
    anscombeUni_H = recanscombe(nsignal(1,:),'Universal','Hard',lev,h);
    cpu_anscombeUni_H(m,1) = cputime -t;

    t=cputime;
    anscombeUni_S = recanscombe(nsignal(1,:),'Universal','Soft',lev,h);
    cpu_anscombeUni_S(m,1) = cputime -t;

    t=cputime;
    anscombeCV_H = recanscombe(nsignal(1,:),'CV','Hard',lev,h);
    cpu_anscombeCV_H(m,1) = cputime -t;

    t=cputime;
    anscombeCV_S = recanscombe(nsignal(1,:),'CV','Soft',lev,h);
    cpu_anscombeCV_S(m,1) = cputime -t;

    t=cputime;
    anscombeMin_TI_H = recanscombeTI(nsignal(1,:),'MinMax','Hard',lev,h);
    cpu_anscombeMin_TI_H(m,1) = cputime -t;

    t=cputime;
    anscombeMin_TI_S = recanscombeTI(nsignal(1,:),'MinMax','Soft',lev,h);
    cpu_anscombeMin_TI_S(m,1) = cputime -t;

    t=cputime;
    anscombeUni_TI_H = recanscombeTI(nsignal(1,:),'Universal','Hard',lev,h);
    cpu_anscombeUni_TI_H(m,1) = cputime -t;

    t=cputime;
    anscombeUni_TI_S = recanscombeTI(nsignal(1,:),'Universal','Soft',lev,h);
    cpu_anscombeUni_TI_S(m,1) = cputime -t;

    t=cputime;
    anscombeCV_TI_H = recanscombeTI(nsignal(1,:),'CV','Hard',lev,h);
    cpu_anscombeCV_TI_H(m,1) = cputime -t;

    t=cputime;
    anscombeCV_TI_S = recanscombeTI(nsignal(1,:),'CV','Soft',lev,h);
    cpu_anscombeCV_TI_S(m,1) = cputime -t;

    t=cputime;
    fiszMin_H = recfisz(nsignal(1,:),'MinMax','Hard',lev,h);
    cpu_fiszMin_H(m,1) = cputime -t;

    t=cputime;
    fiszMin_S = recfisz(nsignal(1,:),'MinMax','Soft',lev,h);
    cpu_fiszMin_S(m,1) = cputime -t;

    t=cputime;
    fiszUni_H = recfisz(nsignal(1,:),'Universal','Hard',lev,h);
    cpu_fiszUni_H(m,1) = cputime -t;

    t=cputime;
    fiszUni_S = recfisz(nsignal(1,:),'Universal','Soft',lev,h);
    cpu_fiszUni_S(m,1) = cputime -t;

    t=cputime;
    fiszCV_H = recfisz(nsignal(1,:),'CV','Hard',lev,h);
    cpu_fiszCV_H(m,1) = cputime -t;

    t=cputime;
    fiszCV_S = recfisz(nsignal(1,:),'CV','Soft',lev,h);
    cpu_fiszCV_S(m,1) = cputime -t;

    t=cputime;
    haar_TI_H = rechaarTI(nsignal(1,:),scale,'Hard',lev);
    cpu_haar_TI_H(m,1) = cputime -t;
 
    t=cputime;
    haar_TI_S = rechaarTI(nsignal(1,:),scale,'Soft',lev);
    cpu_haar_TI_S(m,1) = cputime -t;
 
    t=cputime;
    corrected_H = reccorrected(nsignal(1,:),scale,ordind,'Hard',lev,h);
    cpu_corrected_H(m,1) = cputime -t;

    t=cputime;
    corrected_S = reccorrected(nsignal(1,:),scale,ordind,'Soft',lev,h);
    cpu_corrected_S(m,1) = cputime -t;

    t=cputime;
    corrected_TI_H = reccorrectedTI(nsignal(1,:),scale,ordind,'Hard',lev,h);
    cpu_corrected_TI_H(m,1) = cputime -t;

    t=cputime;
    corrected_TI_S = reccorrectedTI(nsignal(1,:),scale,ordind,'Soft',lev,h);
    cpu_corrected_TI_S(m,1) = cputime -t;
 
    t=cputime;
    modulation = recmodulation(nsignal(1,:),lev,h);
    cpu_modulation(m,1) = cputime -t;

    pqind=1;
    t=cputime;
    [BMSMShrink_TI,ppp,qqq] = BMSMShrink(nsignal(1,:),pqind);
    cpu_BMSMShrink_TI(m,1) = cputime -t;

    t=cputime;
    MMIM_TI = BMIE_TI(nsignal(1,:),lev);
    cpu_MMIM_TI(m,1) = cputime -t;
    
    t=cputime;
    distribution='poisson';
    wtrans='FWT_PO';
    wname = 'db';
    NL=3;
    max_IP_iter=10;
    IP_optimal_tol=0.02;
    feasible_tol=0.02;
    max_CG_iter=2*length(nsignal);
    CG_tol=0.01;
    lambda=0;
    PENl1 = GBP(nsignal(1,:),lambda,distribution,wtrans,wname,NL,max_IP_iter,...
    IP_optimal_tol,feasible_tol,max_CG_iter,CG_tol,0,0,0,0,0,0);
    cpu_PENl1(m,1) = cputime -t;
    
%%%%%%%%%
    fprintf('%4i%',m); % simulation number
    fprintf('\n');

%calculate the Mean Squared Error
    mse_anscombeMin_H(m,1) = nanmean((anscombeMin_H-signal).^2);
    mse_anscombeMin_S(m,1) = nanmean((anscombeMin_S-signal).^2);
    mse_anscombeUni_H(m,1) = nanmean((anscombeUni_H-signal).^2);
    mse_anscombeUni_S(m,1) = nanmean((anscombeUni_S-signal).^2);
    mse_anscombeCV_H(m,1) = nanmean((anscombeCV_H-signal).^2);
    mse_anscombeCV_S(m,1) = nanmean((anscombeCV_S-signal).^2);

    mse_anscombeMin_TI_H(m,1) = nanmean((anscombeMin_TI_H-signal).^2);
    mse_anscombeMin_TI_S(m,1) = nanmean((anscombeMin_TI_S-signal).^2);
    mse_anscombeUni_TI_H(m,1) = nanmean((anscombeUni_TI_H-signal).^2);
    mse_anscombeUni_TI_S(m,1) = nanmean((anscombeUni_TI_S-signal).^2);
    mse_anscombeCV_TI_H(m,1) = nanmean((anscombeCV_TI_H-signal).^2);
    mse_anscombeCV_TI_S(m,1) = nanmean((anscombeCV_TI_S-signal).^2);

    mse_fiszMin_H(m,1) = nanmean((fiszMin_H-signal).^2);
    mse_fiszMin_S(m,1) = nanmean((fiszMin_S-signal).^2);
    mse_fiszUni_H(m,1) = nanmean((fiszUni_H-signal).^2);
    mse_fiszUni_S(m,1) = nanmean((fiszUni_S-signal).^2);
    mse_fiszCV_H(m,1) = nanmean((fiszCV_H-signal).^2);
    mse_fiszCV_S(m,1) = nanmean((fiszCV_S-signal).^2);

    mse_haar_TI_H(m,1) = nanmean((haar_TI_H-signal).^2);
    mse_haar_TI_S(m,1) = nanmean((haar_TI_S-signal).^2);

    mse_corrected_H(m,1) = nanmean((corrected_H-signal).^2);
    mse_corrected_S(m,1) = nanmean((corrected_S-signal).^2);

    mse_corrected_TI_H(m,1) = nanmean((corrected_TI_H-signal).^2);
    mse_corrected_TI_S(m,1) = nanmean((corrected_TI_S-signal).^2);

    mse_modulation(m,1) = nanmean((modulation-signal).^2);

    mse_BMSMShrink_TI(m,1) = nanmean((BMSMShrink_TI-signal).^2);
    
    mse_MMIM_TI(m,1) = nanmean((MMIM_TI-signal).^2);
    
    mse_PENl1(m,1) = nanmean((PENl1-signal).^2);

%stack the various estimates
    all_anscombeMin_H = all_anscombeMin_H + anscombeMin_H;
    all_anscombeMin_S = all_anscombeMin_S + anscombeMin_S;
    all_anscombeUni_H = all_anscombeUni_H + anscombeUni_H;
    all_anscombeUni_S = all_anscombeUni_S + anscombeUni_S;
    all_anscombeCV_H = all_anscombeCV_H + anscombeCV_H;
    all_anscombeCV_S = all_anscombeCV_S + anscombeCV_S;

    all_anscombeMin_TI_H = all_anscombeMin_TI_H + anscombeMin_TI_H;
    all_anscombeMin_TI_S = all_anscombeMin_TI_S + anscombeMin_TI_S;
    all_anscombeUni_TI_H = all_anscombeUni_TI_H + anscombeUni_TI_H;
    all_anscombeUni_TI_S = all_anscombeUni_TI_S + anscombeUni_TI_S;
    all_anscombeCV_TI_H = all_anscombeCV_TI_H + anscombeCV_TI_H;
    all_anscombeCV_TI_S = all_anscombeCV_TI_S + anscombeCV_TI_S;

    all_fiszMin_H = all_fiszMin_H + fiszMin_H;
    all_fiszMin_S = all_fiszMin_S + fiszMin_S;
    all_fiszUni_H = all_fiszUni_H + fiszUni_H;
    all_fiszUni_S = all_fiszUni_S + fiszUni_S;
    all_fiszCV_H = all_fiszCV_H + fiszCV_H;
    all_fiszCV_S = all_fiszCV_S + fiszCV_S;
 
    all_haar_TI_H = all_haar_TI_H + haar_TI_H;
    all_haar_TI_S = all_haar_TI_S + haar_TI_S;

    all_corrected_H = all_corrected_H + corrected_H;
    all_corrected_S = all_corrected_S + corrected_S;

    all_corrected_TI_H = all_corrected_TI_H + corrected_TI_H;
    all_corrected_TI_S = all_corrected_TI_S + corrected_TI_S;

    all_modulation = all_modulation + modulation;

    all_BMSMShrink_TI = all_BMSMShrink_TI + BMSMShrink_TI;

    all_MMIM_TI = all_MMIM_TI + MMIM_TI;

    all_PENl1 = all_PENl1 + PENl1;

%%%%%%%%%
    all_anscombeMin_H2 = all_anscombeMin_H2 + anscombeMin_H.^2;
    all_anscombeMin_S2 = all_anscombeMin_S2 + anscombeMin_S.^2;
    all_anscombeUni_H2 = all_anscombeUni_H2 + anscombeUni_H.^2;
    all_anscombeUni_S2 = all_anscombeUni_S2 + anscombeUni_S.^2;
    all_anscombeCV_H2 = all_anscombeCV_H2 + anscombeCV_H.^2;
    all_anscombeCV_S2 = all_anscombeCV_S2 + anscombeCV_S.^2;

    all_anscombeMin_TI_H2 = all_anscombeMin_TI_H2 + anscombeMin_TI_H.^2;
    all_anscombeMin_TI_S2 = all_anscombeMin_TI_S2 + anscombeMin_TI_S.^2;
    all_anscombeUni_TI_H2 = all_anscombeUni_TI_H2 + anscombeUni_TI_H.^2;
    all_anscombeUni_TI_S2 = all_anscombeUni_TI_S2 + anscombeUni_TI_S.^2;
    all_anscombeCV_TI_H2 = all_anscombeCV_TI_H2 + anscombeCV_TI_H.^2;
    all_anscombeCV_TI_S2 = all_anscombeCV_TI_S2 + anscombeCV_TI_S.^2;

    all_fiszMin_H2 = all_fiszMin_H2 + fiszMin_H.^2;
    all_fiszMin_S2 = all_fiszMin_S2 + fiszMin_S.^2;
    all_fiszUni_H2 = all_fiszUni_H2 + fiszUni_H.^2;
    all_fiszUni_S2 = all_fiszUni_S2 + fiszUni_S.^2;
    all_fiszCV_H2 = all_fiszCV_H2 + fiszCV_H.^2;
    all_fiszCV_S2 = all_fiszCV_S2 + fiszCV_S.^2;
 
    all_haar_TI_H2 = all_haar_TI_H2 + haar_TI_H.^2;
    all_haar_TI_S2 = all_haar_TI_S2 + haar_TI_S.^2;

    all_corrected_H2 = all_corrected_H2 + corrected_H.^2;
    all_corrected_S2 = all_corrected_S2 + corrected_S.^2;

    all_corrected_TI_H2 = all_corrected_TI_H2 + corrected_TI_H.^2;
    all_corrected_TI_S2 = all_corrected_TI_S2 + corrected_TI_S.^2;

    all_modulation_2 = all_modulation_2 + modulation.^2;

    all_BMSMShrink_TI_2 = all_BMSMShrink_TI_2 + BMSMShrink_TI.^2;

    all_MMIM_TI_2 = all_MMIM_TI_2 + MMIM_TI.^2;
    
    all_PENl1_2 = all_PENl1_2 + PENl1.^2;
    
%calculate the Maximum Deviation
    mxdv_anscombeMin_H(m,1) = nanmax(abs(anscombeMin_H-signal));
    mxdv_anscombeMin_S(m,1) = nanmax(abs(anscombeMin_S-signal));
    mxdv_anscombeUni_H(m,1) = nanmax(abs(anscombeUni_H-signal));
    mxdv_anscombeUni_S(m,1) = nanmax(abs(anscombeUni_S-signal));
    mxdv_anscombeCV_H(m,1) = nanmax(abs(anscombeCV_H-signal));
    mxdv_anscombeCV_S(m,1) = nanmax(abs(anscombeCV_S-signal));

    mxdv_anscombeMin_TI_H(m,1) = nanmax(abs(anscombeMin_TI_H-signal));
    mxdv_anscombeMin_TI_S(m,1) = nanmax(abs(anscombeMin_TI_S-signal));
    mxdv_anscombeUni_TI_H(m,1) = nanmax(abs(anscombeUni_TI_H-signal));
    mxdv_anscombeUni_TI_S(m,1) = nanmax(abs(anscombeUni_TI_S-signal));
    mxdv_anscombeCV_TI_H(m,1) = nanmax(abs(anscombeCV_TI_H-signal));
    mxdv_anscombeCV_TI_S(m,1) = nanmax(abs(anscombeCV_TI_S-signal));

    mxdv_fiszMin_H(m,1) = nanmax(abs(fiszMin_H-signal));
    mxdv_fiszMin_S(m,1) = nanmax(abs(fiszMin_S-signal));
    mxdv_fiszUni_H(m,1) = nanmax(abs(fiszUni_H-signal));
    mxdv_fiszUni_S(m,1) = nanmax(abs(fiszUni_S-signal));
    mxdv_fiszCV_H(m,1) = nanmax(abs(fiszCV_H-signal));
    mxdv_fiszCV_S(m,1) = nanmax(abs(fiszCV_S-signal));

    mxdv_haar_TI_H(m,1) = nanmax(abs(haar_TI_H-signal));
    mxdv_haar_TI_S(m,1) = nanmax(abs(haar_TI_S-signal));

    mxdv_corrected_H(m,1) = max(abs(corrected_H-signal));
    mxdv_corrected_S(m,1) = max(abs(corrected_S-signal));

    mxdv_corrected_TI_H(m,1) = max(abs(corrected_TI_H-signal));
    mxdv_corrected_TI_S(m,1) = max(abs(corrected_TI_S-signal));

    mxdv_modulation(m,1) = nanmax(abs(modulation-signal));

    mxdv_BMSMShrink_TI(m,1) = nanmax(abs(BMSMShrink_TI-signal));

    mxdv_MMIM_TI(m,1) = nanmax(abs(MMIM_TI-signal));

    mxdv_PENl1(m,1) = nanmax(abs(PENl1-signal));
    
%calculate the L1 Norm
    l1_anscombeMin_H(m,1) = nansum(abs(anscombeMin_H-signal));
    l1_anscombeMin_S(m,1) = nansum(abs(anscombeMin_S-signal));
    l1_anscombeUni_H(m,1) = nansum(abs(anscombeUni_H-signal));
    l1_anscombeUni_S(m,1) = nansum(abs(anscombeUni_S-signal));
    l1_anscombeCV_H(m,1) = nansum(abs(anscombeCV_H-signal));
    l1_anscombeCV_S(m,1) = nansum(abs(anscombeCV_S-signal));

    l1_anscombeMin_TI_H(m,1) = nansum(abs(anscombeMin_TI_H-signal));
    l1_anscombeMin_TI_S(m,1) = nansum(abs(anscombeMin_TI_S-signal));
    l1_anscombeUni_TI_H(m,1) = nansum(abs(anscombeUni_TI_H-signal));
    l1_anscombeUni_TI_S(m,1) = nansum(abs(anscombeUni_TI_S-signal));
    l1_anscombeCV_TI_H(m,1) = nansum(abs(anscombeCV_TI_H-signal));
    l1_anscombeCV_TI_S(m,1) = nansum(abs(anscombeCV_TI_S-signal));

    l1_fiszMin_H(m,1) = nansum(abs(fiszMin_H-signal));
    l1_fiszMin_S(m,1) = nansum(abs(fiszMin_S-signal));
    l1_fiszUni_H(m,1) = nansum(abs(fiszUni_H-signal));
    l1_fiszUni_S(m,1) = nansum(abs(fiszUni_S-signal));
    l1_fiszCV_H(m,1) = nansum(abs(fiszCV_H-signal));
    l1_fiszCV_S(m,1) = nansum(abs(fiszCV_S-signal));

    l1_haar_TI_H(m,1) = nansum(abs(haar_TI_H-signal));
    l1_haar_TI_S(m,1) = nansum(abs(haar_TI_S-signal));

    l1_corrected_H(m,1) = nansum(abs(corrected_H-signal));
    l1_corrected_S(m,1) = nansum(abs(corrected_S-signal));

    l1_corrected_TI_H(m,1) = nansum(abs(corrected_TI_H-signal));
    l1_corrected_TI_S(m,1) = nansum(abs(corrected_TI_S-signal));

    l1_modulation(m,1) = nansum(abs(modulation-signal));

    l1_BMSMShrink_TI(m,1) = nansum(abs(BMSMShrink_TI-signal));

    l1_MMIM_TI(m,1) = nansum(abs(MMIM_TI-signal));
    
    l1_PENl1(m,1) = nansum(abs(PENl1-signal));
end

%calculate the Root Mean Squared Error
rmse_anscombeMin_H = sqrt(nanmean(mse_anscombeMin_H));
rmse_anscombeMin_S = sqrt(nanmean(mse_anscombeMin_S));
rmse_anscombeUni_H = sqrt(nanmean(mse_anscombeUni_H));
rmse_anscombeUni_S = sqrt(nanmean(mse_anscombeUni_S));
rmse_anscombeCV_H = sqrt(nanmean(mse_anscombeCV_H));
rmse_anscombeCV_S = sqrt(nanmean(mse_anscombeCV_S));

rmse_anscombeMin_TI_H = sqrt(nanmean(mse_anscombeMin_TI_H));
rmse_anscombeMin_TI_S = sqrt(nanmean(mse_anscombeMin_TI_S));
rmse_anscombeUni_TI_H = sqrt(nanmean(mse_anscombeUni_TI_H));
rmse_anscombeUni_TI_S = sqrt(nanmean(mse_anscombeUni_TI_S));
rmse_anscombeCV_TI_H = sqrt(nanmean(mse_anscombeCV_TI_H));
rmse_anscombeCV_TI_S = sqrt(nanmean(mse_anscombeCV_TI_S));

rmse_fiszMin_H = sqrt(nanmean(mse_fiszMin_H));
rmse_fiszMin_S = sqrt(nanmean(mse_fiszMin_S));
rmse_fiszUni_H = sqrt(nanmean(mse_fiszUni_H));
rmse_fiszUni_S = sqrt(nanmean(mse_fiszUni_S));
rmse_fiszCV_H = sqrt(nanmean(mse_fiszCV_H));
rmse_fiszCV_S = sqrt(nanmean(mse_fiszCV_S));

rmse_haar_TI_H = sqrt(nanmean(mse_haar_TI_H));
rmse_haar_TI_S = sqrt(nanmean(mse_haar_TI_S));

rmse_corrected_H = sqrt(nanmean(mse_corrected_H));
rmse_corrected_S = sqrt(nanmean(mse_corrected_S));

rmse_corrected_TI_H = sqrt(nanmean(mse_corrected_TI_H));
rmse_corrected_TI_S = sqrt(nanmean(mse_corrected_TI_S));

rmse_modulation = sqrt(nanmean(mse_modulation));

rmse_BMSMShrink_TI = sqrt(nanmean(mse_BMSMShrink_TI));

rmse_MMIM_TI = sqrt(nanmean(mse_MMIM_TI));

rmse_PENl1 = sqrt(nanmean(mse_PENl1));

%calculate the Root Mean Squared Bias
m_anscombeMin_H = all_anscombeMin_H/M;
m_anscombeMin_S = all_anscombeMin_S/M;
m_anscombeUni_H = all_anscombeUni_H/M;
m_anscombeUni_S = all_anscombeUni_S/M;
m_anscombeCV_H = all_anscombeCV_H/M;
m_anscombeCV_S = all_anscombeCV_S/M;

m_anscombeMin_TI_H = all_anscombeMin_TI_H/M;
m_anscombeMin_TI_S = all_anscombeMin_TI_S/M;
m_anscombeUni_TI_H = all_anscombeUni_TI_H/M;
m_anscombeUni_TI_S = all_anscombeUni_TI_S/M;
m_anscombeCV_TI_H = all_anscombeCV_TI_H/M;
m_anscombeCV_TI_S = all_anscombeCV_TI_S/M;

m_fiszMin_H = all_fiszMin_H/M;
m_fiszMin_S = all_fiszMin_S/M;
m_fiszUni_H = all_fiszUni_H/M;
m_fiszUni_S = all_fiszUni_S/M;
m_fiszCV_H = all_fiszCV_H/M;
m_fiszCV_S = all_fiszCV_S/M;

m_haar_TI_H = all_haar_TI_H/M;
m_haar_TI_S = all_haar_TI_S/M;

m_corrected_H = all_corrected_H/M;
m_corrected_S = all_corrected_S/M;

m_corrected_TI_H = all_corrected_TI_H/M;
m_corrected_TI_S = all_corrected_TI_S/M;

m_modulation = all_modulation/M;

m_BMSMShrink_TI = all_BMSMShrink_TI/M;

m_MMIM_TI = all_MMIM_TI/M;

m_PENl1 = all_PENl1/M;

%%%%%%%%%%%%%%%%%
rmsb_anscombeMin_H = sqrt(nanmean((m_anscombeMin_H-signal).^2));
rmsb_anscombeMin_S = sqrt(nanmean((m_anscombeMin_S-signal).^2));
rmsb_anscombeUni_H = sqrt(nanmean((m_anscombeUni_H-signal).^2));
rmsb_anscombeUni_S = sqrt(nanmean((m_anscombeUni_S-signal).^2));
rmsb_anscombeCV_H = sqrt(nanmean((m_anscombeCV_H-signal).^2));
rmsb_anscombeCV_S = sqrt(nanmean((m_anscombeCV_S-signal).^2));

rmsb_anscombeMin_TI_H = sqrt(nanmean((m_anscombeMin_TI_H-signal).^2));
rmsb_anscombeMin_TI_S = sqrt(nanmean((m_anscombeMin_TI_S-signal).^2));
rmsb_anscombeUni_TI_H = sqrt(nanmean((m_anscombeUni_TI_H-signal).^2));
rmsb_anscombeUni_TI_S = sqrt(nanmean((m_anscombeUni_TI_S-signal).^2));
rmsb_anscombeCV_TI_H = sqrt(nanmean((m_anscombeCV_TI_H-signal).^2));
rmsb_anscombeCV_TI_S = sqrt(nanmean((m_anscombeCV_TI_S-signal).^2));

rmsb_fiszMin_H = sqrt(nanmean((m_fiszMin_H-signal).^2));
rmsb_fiszMin_S = sqrt(nanmean((m_fiszMin_S-signal).^2));
rmsb_fiszUni_H = sqrt(nanmean((m_fiszUni_H-signal).^2));
rmsb_fiszUni_S = sqrt(nanmean((m_fiszUni_S-signal).^2));
rmsb_fiszCV_H = sqrt(nanmean((m_fiszCV_H-signal).^2));
rmsb_fiszCV_S = sqrt(nanmean((m_fiszCV_S-signal).^2));

rmsb_haar_TI_H = sqrt(nanmean((m_haar_TI_H-signal).^2));
rmsb_haar_TI_S = sqrt(nanmean((m_haar_TI_S-signal).^2));

rmsb_corrected_H = sqrt(nanmean((m_corrected_H-signal).^2));
rmsb_corrected_S = sqrt(nanmean((m_corrected_S-signal).^2));

rmsb_corrected_TI_H = sqrt(nanmean((m_corrected_TI_H-signal).^2));
rmsb_corrected_TI_S = sqrt(nanmean((m_corrected_TI_S-signal).^2));

rmsb_modulation = sqrt(nanmean((m_modulation-signal).^2));

rmsb_BMSMShrink_TI = sqrt(nanmean((m_BMSMShrink_TI-signal).^2));

rmsb_MMIM_TI = sqrt(nanmean((m_MMIM_TI-signal).^2));

rmsb_PENl1 = sqrt(nanmean((m_PENl1-signal).^2));

%stack the Mean Squared Error Vectors
mse = [mse_anscombeMin_H mse_anscombeMin_S mse_anscombeUni_H mse_anscombeUni_S mse_anscombeCV_H ...
        mse_anscombeCV_S mse_anscombeMin_TI_H mse_anscombeMin_TI_S mse_anscombeUni_TI_H ...
        mse_anscombeUni_TI_S mse_anscombeCV_TI_H mse_anscombeCV_TI_S mse_fiszMin_H mse_fiszMin_S ...
        mse_fiszUni_H mse_fiszUni_S mse_fiszCV_H mse_fiszCV_S mse_haar_TI_H mse_haar_TI_S ...
        mse_corrected_H mse_corrected_S mse_corrected_TI_H mse_corrected_TI_S mse_modulation mse_BMSMShrink_TI...
        mse_MMIM_TI mse_PENl1];       
     
%stack the Maximum Deviation Vectors
mxdv = [mxdv_anscombeMin_H mxdv_anscombeMin_S mxdv_anscombeUni_H mxdv_anscombeUni_S mxdv_anscombeCV_H ...
        mxdv_anscombeCV_S mxdv_anscombeMin_TI_H mxdv_anscombeMin_TI_S mxdv_anscombeUni_TI_H ...
        mxdv_anscombeUni_TI_S mxdv_anscombeCV_TI_H mxdv_anscombeCV_TI_S mxdv_fiszMin_H mxdv_fiszMin_S ...
        mxdv_fiszUni_H mxdv_fiszUni_S mxdv_fiszCV_H mxdv_fiszCV_S mxdv_haar_TI_H mxdv_haar_TI_S ...
        mxdv_corrected_H mxdv_corrected_S mxdv_corrected_TI_H mxdv_corrected_TI_S mxdv_modulation mxdv_BMSMShrink_TI...
        mxdv_MMIM_TI mxdv_PENl1]; 
	
%stack the L1 Norm Vectors
l1norm = [l1_anscombeMin_H l1_anscombeMin_S l1_anscombeUni_H l1_anscombeUni_S l1_anscombeCV_H ...
          l1_anscombeCV_S l1_anscombeMin_TI_H l1_anscombeMin_TI_S l1_anscombeUni_TI_H ...
          l1_anscombeUni_TI_S l1_anscombeCV_TI_H l1_anscombeCV_TI_S l1_fiszMin_H l1_fiszMin_S ...
          l1_fiszUni_H l1_fiszUni_S l1_fiszCV_H l1_fiszCV_S l1_haar_TI_H l1_haar_TI_S l1_corrected_H ...
          l1_corrected_S l1_corrected_TI_H l1_corrected_TI_S l1_modulation l1_BMSMShrink_TI l1_MMIM_TI l1_PENl1];           

%stack the CPU time Vectors
cpu = [cpu_anscombeMin_H cpu_anscombeMin_S cpu_anscombeUni_H cpu_anscombeUni_S cpu_anscombeCV_H ...
       cpu_anscombeCV_S cpu_anscombeMin_TI_H cpu_anscombeMin_TI_S cpu_anscombeUni_TI_H ...
       cpu_anscombeUni_TI_S cpu_anscombeCV_TI_H cpu_anscombeCV_TI_S cpu_fiszMin_H cpu_fiszMin_S ...
       cpu_fiszUni_H cpu_fiszUni_S cpu_fiszCV_H cpu_fiszCV_S cpu_haar_TI_H cpu_haar_TI_S cpu_corrected_H ...
       cpu_corrected_S cpu_corrected_TI_H cpu_corrected_TI_S cpu_modulation cpu_BMSMShrink_TI cpu_MMIM_TI cpu_PENl1];
       
%%%%%%%%%%%%%%
nest=size(mse,2);
if( M == 1)
    stdmse=zeros(nest,1);
    amse_f = [mse' stdmse];
    %
    stdmxdv=zeros(nest,1);
    amxdv_f = [mxdv' stdmxdv];
    %
    stdl1norm=zeros(nest,1);
    al1norm_f = [l1norm' stdl1norm];
    %
    stdcpu=zeros(nest,1);
    acputime_f=[cpu' stdcpu];
else
    amse_f = [nanmean(mse)' nanstd(mse)'];
    amxdv_f = [nanmean(mxdv)' nanstd(mxdv)'];
    al1norm_f = [nanmean(l1norm)' nanstd(l1norm)'];
    acputime_f = [nanmean(cpu)' nanstd(cpu)'];
end

%rmse_f : Root Mean Squarred Error
rmse_f = [rmse_anscombeMin_H rmse_anscombeMin_S rmse_anscombeUni_H rmse_anscombeUni_S rmse_anscombeCV_H ...
          rmse_anscombeCV_S rmse_anscombeMin_TI_H rmse_anscombeMin_TI_S rmse_anscombeUni_TI_H ...
          rmse_anscombeUni_TI_S rmse_anscombeCV_TI_H rmse_anscombeCV_TI_S rmse_fiszMin_H rmse_fiszMin_S ...
          rmse_fiszUni_H rmse_fiszUni_S rmse_fiszCV_H rmse_fiszCV_S rmse_haar_TI_H rmse_haar_TI_S ...
          rmse_corrected_H rmse_corrected_S rmse_corrected_TI_H rmse_corrected_TI_S rmse_modulation rmse_BMSMShrink_TI...
          rmse_MMIM_TI rmse_PENl1];       
       
%rmsb_f : Root Mean Squarred Bias
rmsb_f = [rmsb_anscombeMin_H rmsb_anscombeMin_S rmsb_anscombeUni_H rmsb_anscombeUni_S rmsb_anscombeCV_H ...
          rmsb_anscombeCV_S rmsb_anscombeMin_TI_H rmsb_anscombeMin_TI_S rmsb_anscombeUni_TI_H ...
          rmsb_anscombeUni_TI_S rmsb_anscombeCV_TI_H rmsb_anscombeCV_TI_S rmsb_fiszMin_H rmsb_fiszMin_S ...
          rmsb_fiszUni_H rmsb_fiszUni_S rmsb_fiszCV_H rmsb_fiszCV_S rmsb_haar_TI_H rmsb_haar_TI_S ...
          rmsb_corrected_H rmsb_corrected_S rmsb_corrected_TI_H rmsb_corrected_TI_S rmsb_modulation rmsb_BMSMShrink_TI...
          rmsb_MMIM_TI rmsb_PENl1];       
          
%%%%%%%%%%%%
if show == 1,
   
%Producing the Box-Plots and Bar-Plots  
    figure(1)
    clf;
    subplot('Position',[0.03 0.73 0.44 0.17])
    boxplot(mse);
    set(gca,'XTick',[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28],'FontSize',6) 
    ylabel('')
    xlabel('')
    title(['MSE - ',char(names(test))],'FontSize',8)
    
    subplot('Position',[0.54 0.73 0.44 0.17])
    boxplot(l1norm);
    set(gca,'XTick',[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28],'FontSize',6) 
    ylabel('') 
    xlabel('')
    title(['L1 - ',char(names(test))],'FontSize',8)
    
    subplot('Position',[0.03 0.45 0.44 0.17])
    bar(rmse_f,0.7);
    set(gca,'XTick',[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28],'FontSize',6) 
    axis([0 29 -Inf Inf])
    ylabel('')
    xlabel('')
    title(['RMSE - ',char(names(test))],'FontSize',8)
    
    msb_f=rmsb_f.^2;
    mse_f=rmse_f.^2;
    sb_mse=msb_f./mse_f;
    subplot('Position',[0.54 0.45 0.44 0.17])
    bar(sb_mse,0.7);
    set(gca,'XTick',[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28],'FontSize',6) 
    axis([0 29 -Inf Inf])
    ylabel('')
    xlabel('')
    title(['SB/MSE - ',char(names(test))],'FontSize',8)
    
    subplot('Position',[0.03 0.17 0.44 0.17])
    boxplot(mxdv);
    set(gca,'XTick',[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28],'FontSize',6) 
    ylabel('')
    xlabel('')
    title(['MXDV - ',char(names(test))],'FontSize',8)
    
    subplot('Position',[0.54 0.17 0.44 0.17])
    boxplot(cpu);
    set(gca,'XTick',[1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28],'FontSize',6) 
    ylabel('')
    xlabel('')
    title(['CPU - ',char(names(test))],'FontSize',8)
    gname=sprintf('M%it%in%is%io%ih%il%iALLMEASURES',M,test,n,scale,ordind,hint,lev);
    printFigure(2,gname)
    
%%%%%%%%%
xt=(1:n)/n;
%%%%%%%%%
    
    figure(2)
    clf;
    subplot(7,2,1);
    plot(xt,m_anscombeMin_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeMin-H - ',char(names(test))],'FontSize',8);
        
    subplot(7,2,2);
    plot(xt,m_anscombeMin_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeMin-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,3);
    plot(xt,m_anscombeUni_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeUni-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,4);
    plot(xt,m_anscombeUni_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeUni-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,5);
    plot(xt,m_anscombeCV_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeCV-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,6);
    plot(xt,m_anscombeCV_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeCV-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,7);
    plot(xt,m_anscombeMin_TI_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeMin-TI-H - ',char(names(test))],'FontSize',8);
        
    subplot(7,2,8);
    plot(xt,m_anscombeMin_TI_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeMin-TI-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,9);
    plot(xt,m_anscombeUni_TI_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeUni-TI-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,10);
    plot(xt,m_anscombeUni_TI_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeUni-TI-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,11);
    plot(xt,m_anscombeCV_TI_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeCV-TI-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,12);
    plot(xt,m_anscombeCV_TI_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['AnscombeCV-TI-S - ',char(names(test))],'FontSize',8);
  
    subplot(7,2,13);
    plot(xt,m_fiszMin_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['FiszMin-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,14);
    plot(xt,m_fiszMin_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['FiszMin-S - ',char(names(test))],'FontSize',8);
    gname=sprintf('M%it%in%is%io%ih%il%iAVE1',M,test,n,scale,ordind,hint,lev);
    printFigure(2,gname)
    
 %%%%%%%%%%    
    figure(3)
    clf;
    subplot(7,2,1);
    plot(xt,m_fiszUni_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['FiszUni-H - ',char(names(test))],'FontSize',8);
   
    subplot(7,2,2);
    plot(xt,m_fiszUni_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['FiszUni-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,3);
    plot(xt,m_fiszCV_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['FiszCV-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,4);
    plot(xt,m_fiszCV_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['FiszCV-S - ',char(names(test))],'FontSize',8);
    
%%%%%%%%%
    subplot(7,2,5);
    plot(xt,m_haar_TI_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['Haar-TI-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,6);
    plot(xt,m_haar_TI_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['Haar-TI-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,7);
    plot(xt,m_corrected_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['Corrected-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,8);
    plot(xt,m_corrected_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['Corrected-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,9);
    plot(xt,m_corrected_TI_H,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['Corrected-TI-H - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,10);
    plot(xt,m_corrected_TI_S,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['Corrected-TI-S - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,11)
    plot(xt,m_modulation,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['Modulation - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,12)
    plot(xt,m_BMSMShrink_TI,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['BMSM-TI - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,13)
    plot(xt,m_MMIM_TI,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['BMMIM-TI - ',char(names(test))],'FontSize',8);
    
    subplot(7,2,14)
    plot(xt,m_PENl1,xt,signal);
    set(gca,'XTick',[0 0.5 1],'Fontsize',8)
    title(['GBP - ',char(names(test))],'FontSize',8);
    gname=sprintf('M%it%in%is%io%ih%il%iAVE2',M,test,n,scale,ordind,hint,lev);
    printFigure(2,gname)
%%%%%%%
end % end 'if show == 1'
%%%%%%%

fprintf('\n')
info.rnames = strvcat('Rows','anscombeMin_H','anscombeMin_S','anscombeUni_H','anscombeUni_S',...
                      'anscombeCV_H','anscombeCV_S','anscombeMin_TI_H','anscombeMin_TI_S','anscombeUni_TI_H',...
                      'anscombeUni_TI_S','anscombeCV_TI_H','anscombeCV_TI_S','fiszMin_H','fiszMin_S','fiszUni_H',...
                      'fiszUni_S','fiszCV_H','fiszCV_S','haar_TI_H','haar_TI_S','corrected_H','corrected_S',...
                      'corrected_TI_H','corrected_TI_S','modulation','SI-Bayesian Multiscale',...
                      'SI-Multiscale Multiplicative Innovations','GBP');
resultats=[amse_f amxdv_f al1norm_f acputime_f rmse_f' rmsb_f'];
[info.endr,info.endc]=size(resultats);
info.cnames = strvcat('Average MSE','Standard Deviation','Average MXDV','Standard Deviation', ...
                      'Average L1norm','Standard Deviation','Average CPU','Standard Deviation','RMSE','RMSB');
info.fid=fidname;
lprint(resultats,info);
fclose(fidname);
f = struct('amse_f', amse_f, 'rmse_f', rmse_f, 'rmsb_f', rmsb_f, 'amxdv_f', amxdv_f, 'al1norm_f', al1norm_f, 'acputime_f', acputime_f); 

% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom.
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus.
%
% mailto: T.Sapatinas@ucy.ac.cy 


