function f = TholdSolve(minj,maxj,lambda0,qmf,dirind,ordind)

% TholdSolve.m:   Crude zero-finding algorithm to solve for threshold constants  
%                 'b' when de-noising Poisson count data using periodic,         
%                 biorthogonal, symmetric wavelets.                             
% Usage
%                 f = TholdSolve(minj,maxj,lambda0,qmf,dirind,ordind)
% Inputs                                                               
%       minj      Minimum resolution level (minj >= 4).                 
%       maxj      Maximum resolution level.                             
%       lambda0   Background intensity level (true value or an estimate).                   
%       qmf       Quadrature mirror filter.                              
%       dirind    Direction Indicator ('min' or 'max').                 
%       ordind    Order Indicator (3 or 4) i.e., correcting             
%                 for just skewness or also kurtosis.                   
% Outputs                                                              
%       f         Vector of b-values for each resolution level,         
%                 ordered (left to right) from maxj to minj.            
% References
%                 Kolaczyk, E.D. (1999). Wavelet shrinkage estimation of certain 
%                 Poisson intensity signals using corrected thresholds. "Statistica 
%                 Sinica", 9, 119-135.

%warning
if minj < 4,
  'SORRY !  Must have minj >= 4 for now.'
end

%calculate integral of 3rd and 4th power of wavelets
n = 2^(maxj+1);
wvec = zeros(1,n);
wvec(24) = 1.0;
L = 4;
wlet = IWT_PO(wvec,L,qmf);
wlet2 = mean(wlet.^2);
wlet3 = mean(wlet.^3); wlet3 = wlet3/(2^(L/2));
wlet4 = mean(wlet.^4); wlet4 = wlet4/(2^L);
tvec = zeros(1,(maxj-minj+1));
maxbval = ceil(2*(log(2^maxj)));
for j = minj:maxj,
    nj = 2^j;
    bvals = [0.001:0.001:maxbval];
    
%calculate correction values of the solution equation
    g1 = wlet3/( (wlet2)^(1.5) );
    g1 = (2^(j/2))*g1/(6*sqrt(lambda0));
    if strcmp(dirind,'min'),
       g1 = -g1;
    end
    g2 = wlet2*wlet4 - 2*( wlet3^2 );
    g2 = g2/(wlet2^3);
    g2 = (2^j)*g2/(8*lambda0);
    
%calculate actual solution equation
    soleq = g1*bvals.^3 - 0.5*bvals.^2 - log(bvals);
    soleq = soleq + log(nj*sqrt(2*log(nj)));
    if ordind == 4,
       soleq = soleq + g2*bvals.^4;
    end
    
%plot(bvals,soleq)
    indvec = find(soleq < 0.01 & soleq > -0.01);
    indlen = length(indvec);
    if (indlen == 0),
       'Thold Solution Equation Has No Zero.'
        tvec(j-minj+1) = NaN;
    elseif (rem(indlen,2)==0),
        tvec(j-minj+1) = bvals(indvec(indlen/2));
    else,
        tvec(j-minj+1) = bvals(indvec((indlen+1)/2));
    end
end
tvec = fliplr(tvec);
tvec = tvec*sqrt(wlet2)*sqrt(lambda0);
f=tvec;

% Acknowledgement
%
% This code is based on a MatLab routine kindly provided by E.D. Kolaczyk.
%
% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
