function f = binning(test,channel,nburst,photon_times,num_bins,method,lev,h,ordind,pqind,wname)

% binning:         This routine first bins the RAW data to obtain the Poisson counts 
%                  necessary to implement the various wavelet shrinkage procedures, and then
%                  exercises these procedures on the BATSE data.
% Usage
%                  f = binning(test,channel,nburst,photon_times,num_bins,method,lev,h,scale,ordind,pqind)
% Inputs
%  test            "Integer pointing" to the burst
%			          'burst138', 'burst185', 'burst207', 'burst474', 'burst551', 'burst845'.
%  channel         Channel used to measure the signal.
%  nburst          Number to identify the burst.
%  photon_times    Vector containing the signal: photon arrival times.  
%  num_bins        Number of bins.
%  method          "Integer pointing" to the method chosen to estimate the signal.
%                  (See comment in mainbinning.m)
%  lev             Coarsest resolution level.
%  h               Quadrature mirror filter.
%  ordind          Order indicator dictating whether skewness alone is corrected for, 
%                  or both skewness and kurtosis are corrected for.
%  pqind           defines the method used to elicitate the hyperparameters in BMSM-TI estimator.
%  wname:          defines the wavelet filter to be used in GBP estimator.
%         
% References
%                  Besbeas, P., De Feis, I. & Sapatinas, T. (2002). A comparative
%                  simulation study of wavelet shrinkage estimators for Poisson counts.
%                  Rapporto Tecnico, 240/02, Istituto per le Applicazioni del Calcolo "Mauro
%                  Picone" - Sezione di Napoli, Consiglio Nazionale delle Ricerche, Italy.

%String argument
names = {'AnscombeMin-H' 'AnscombeMin-S' 'AnscombeUni-H' 'AnscombeUni-S' 'AnscombeCV-H' 'AnscombeCV-S'...
'AnscombeMin-TI-H' 'AnscombeMin-TI-S' 'AnscombeUni-TI-H' 'AnscombeUni-TI-S' 'AnscombeCV-TI-H' 'AnscombeCV-TI-S'...
'FiszMin-H' 'FiszMin-S' 'FiszUni-H' 'FiszUni-S' 'FiszCV-H' 'FiszCV-S' 'FiszMin-TI-H' 'FiszMin-TI-S'...
'FiszUni-TI-H' 'FiszUni-TI-S' 'FiszCV-TI-H' 'FiszCV-TI-S' 'Haar-TI-H' 'Haar-TI-S' 'Corrected-H' ...
'Corrected-S' 'Corrected-TI-H' 'Corrected-TI-S' 'Modulation' 'Modulation-TI' 'Multmodulation'... 
'Multmodulation-TI' 'BMSM-TI' 'MMIM-TI' 'GBP'};

% MatLab Histogram requires array specifying bin centers
t_min = min(photon_times);
t_max = max(photon_times);
t_range = t_max - t_min;
dt_bin = t_range / num_bins;
mid_first = t_min + dt_bin/2;
mid_last  = t_max - dt_bin/2;
data_bins = linspace(mid_first,mid_last,num_bins);
poissig = hist(photon_times,data_bins);
asc=data_bins';

 
if 25 <= method & method <= 30
   scale=ceil(mean(poissig(floor(2/5*num_bins)+1:num_bins))); 
end

if method==1 
    poissigest = recanscombe(poissig,'MinMax','Hard',lev,h);
elseif method==2
    poissigest = recanscombe(poissig,'MinMax','Soft',lev,h);
elseif method==3
    poissigest = recanscombe(poissig,'Universal','Hard',lev,h);
elseif method==4
    poissigest = recanscombe(poissig,'Universal','Soft',lev,h);
elseif method==5
    poissigest = recanscombe(poissig,'CV','Hard',lev,h);
elseif method==6
    poissigest = recanscombe(poissig,'CV','Soft',lev,h);
elseif method==7
    poissigest = recanscombeTI(poissig,'MinMax','Hard',lev,h);
elseif method==8
    poissigest = recanscombeTI(poissig,'MinMax','Soft',lev,h);
elseif method==9
    poissigest = recanscombeTI(poissig,'Universal','Hard',lev,h);
elseif method==10
    poissigest = recanscombeTI(poissig,'Universal','Soft',lev,h);
elseif method==11
    poissigest = recanscombeTI(poissig,'CV','Hard',lev,h);
elseif method==12
    poissigest = recanscombeTI(poissig,'CV','Soft',lev,h);
elseif method==13
    poissigest = recfisz(poissig,'MinMax','Hard',lev,h);
elseif method==14
    poissigest = recfisz(poissig,'MinMax','Soft',lev,h);
elseif method==15
    poissigest = recfisz(poissig,'Universal','Hard',lev,h);
elseif method==16
    poissigest = recfisz(poissig,'Universal','Soft',lev,h);
elseif method==17
    poissigest = recfisz(poissig,'CV','Hard',lev,h);
elseif method==18
    poissigest = recfisz(poissig,'CV','Soft',lev,h);
elseif method==19
    poissigest = recfiszTI(poissig,'MinMax','Hard',lev,h);
elseif method==20
    poissigest = recfiszTI(poissig,'MinMax','Soft',lev,h);
elseif method==21
    poissigest = recfiszTI(poissig,'Universal','Hard',lev,h);
elseif method==22
    poissigest = recfiszTI(poissig,'Universal','Soft',lev,h);
elseif method==23
    poissigest = recfiszTI(poissig,'CV','Hard',lev,h);
elseif method==24
    poissigest = recfiszTI(poissig,'CV','Soft',lev,h);
elseif method==25
    poissigest = rechaarTI(poissig,scale,'Hard',lev);
elseif method==26
    poissigest = rechaarTI(poissig,scale,'Soft',lev);
elseif method==27
    poissigest = reccorrected(poissig,scale,ordind,'Hard',lev,h);
elseif method==28
    poissigest = reccorrected(poissig,scale,ordind,'Soft',lev,h);
elseif method==29
    poissigest = reccorrectedTI(poissig,scale,ordind,'Hard',lev,h);
elseif method==30
    poissigest = reccorrectedTI(poissig,scale,ordind,'Soft',lev,h);
elseif method==31
    poissigest = recmodulation(poissig,lev,h);
elseif method==32
    poissigest = recmodulationTI(poissig,lev,h);
elseif method==33
    poissigest = recmultmodulation(poissig,lev,h);
elseif method==34
    poissigest = recmultmodulationTI(poissig,lev,h);
elseif method==35
    [poissigest,ppp,qqq] = BMSMShrink(poissig,pqind);
elseif method==36
   poissigest = BMIE_TI(poissig,lev);
elseif method==37
   distribution='poisson';
   wtrans='FWT_PO';
   NL=3;
   max_IP_iter=10;
   IP_optimal_tol=0.02;
   feasible_tol=0.02;
   max_CG_iter=2*length(poissig);
   CG_tol=0.01;
   lambda=0;
   poissigest = GBP(poissig,lambda,distribution,wtrans,wname,NL,max_IP_iter,...
   IP_optimal_tol,feasible_tol,max_CG_iter,CG_tol,0,0,0,0,0,0); 
end

f=poissigest;

%plotting the results
%changing the values of the xaxis -- seconds
ascsec = asc(1:num_bins)./10^6;

figure(1)
clf;
subplot(2,1,1)
plot(ascsec,poissig,'b')
ylabel('Poisson Counts','FontSize',8)
xlabel('Milliseconds','FontSize',8)
title(['BATSE trigger ', int2str(nburst), ' - energy channel ', int2str(channel)],'FontSize',8)

subplot(2,1,2)
plot(ascsec,poissigest,'g')
ylabel('Poisson Counts','FontSize',8)
xlabel('Milliseconds','FontSize',8)
title([char(names(method)),', ','num-bins = ',int2str(num_bins)],'FontSize',8)

if 1 <= method & method <= 24
   gname=sprintf('Burst%ich%in%im%il%i',nburst,channel,num_bins,method,lev);
end
if method == 25 | method == 26
   gname=sprintf('Burst%ich%in%im%il%is%i',nburst,channel,num_bins,method,lev,scale);
end
if 27 <= method & method <= 30
   gname=sprintf('Burst%ich%in%im%il%is%io%i',nburst,channel,num_bins,method,lev,scale,ordind);
end
if 31 <= method & method <= 34
   gname=sprintf('Burst%ich%in%im%il%i',nburst,channel,num_bins,method,lev);
end
if method == 35
   gname=sprintf('Burst%ich%in%im%ipqind%i',nburst,channel,num_bins,method,pqind);
end
if method == 36
  gname=sprintf('Burst%ich%in%im%i',nburst,channel,num_bins,method,lev);
end
if method ==37
  gname=sprintf('Burst%ich%in%im%i',nburst,channel,num_bins,method);
end

printFigure(2,gname);

% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone"
% Sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 