function f = recanscombeTI(signal,policy,type,lev,h)

% recanscombeTI: Smoothing Poisson counts based on preprocessing the original
%                data using the Anscombe transformation and applying translation invariant 
%                Gaussian-based global thresholds on the resulting empirical wavelet coefficients.       
% Usage
%                f = recanscombeTI(signal,policy,type,lev,h)
% Inputs
%   signal	     1-d Noisy signal, length(signal)= 2^J.
%   policy       'MinMax' if you want the minimax threshold;
%                'Universal' if you want the universal threshold;
%                'CV' if you want thresholding using the 'leave-out-half' 
%                cross-validation strategy. Optional, default = universal threshold.
%   type         'Hard' if you want hard-thresholding; 'Soft' if you want for soft-thresholding. 
%                Optional, default = hard-thresholding.	     
%   lev          Coarsest resolution level; lev << J=log2(length(signal)).
%                Optional, default = 4.
%   h 		     Quadrature mirror filter for wavelet transform.
%		           Optional, default = Symmlet 8.
% Outputs
%   f		        Estimate, obtained by using thresholding on the 
%                preprocessed wavelet coefficients and applying the
%                inverse Anscombe transformation.
% Notes       
%                Other types of Gaussian-based thresholds could also be used.
% References
%                Coifman, R.R. & Donoho, D.L. (1995). Translation-invariant 
%                de-noising. In "Wavelets and Statistics", Antoniadis, A. & 
%                Oppenheim, G. (Eds.), Lect. Notes Statist., 103, pp. 125-150, 
%                New York: Springer-Verlag.
%
%                Donoho, D.L. (1993). Non-linear wavelet methods for recovery of 
%                signals, densities and spectra from indirect and noisy data.
%                In "Proceedings of Symposia in Applied Mathematics: Different 
%                Perspectives on Wavelets", 47, Ed. I. Daubechies, pp. 173-205. 
%                San Antonio: American Mathematical Society.
%
%                Donoho, D.L. & Johnstone, I.M. (1994). Ideal spatial adaptation by 
%                wavelet shrinkage. Biometrika, 81, 425-455.
%
%                Nason, G.P. (1996). Wavelet shrinkage using cross-validation.
%                J. R. Statist. Soc. B, 58, 463-479.

if nargin < 5
	h = MakeONFilter('Symmlet',8);
end
if nargin < 4
	lev = 4;
end
if nargin < 3
	type = 'Hard';
end
if nargin < 2
   policy = 'Universal'
end

%Initialisations
n = length(signal);
J=log2(n);

%Preprocessing the signal using the Anscombe transformation
zsignal = 2*sqrt(signal+3/8);
if strcmp(policy,'MinMax') == 1
   if isempty(intersect(J,[7 8 9 10]))
      fprintf('Warning: length(signal) must be 128, 256, 512 or 1024');
      f = NaN;
      return;
   end
   thrhard = [2.913 3.117 3.312 3.497];
   thrsoft = [1.669 1.859 2.045 2.226];
   
%Choosing the MiniMax threshold
   if strcmp(type,'Hard') == 1
      thr = thrhard(J-6);
   else
      thr = thrsoft(J-6);
   end
   
%Computing the translation-invariant table
   tiwt = FWT_TI(zsignal,lev,h);
   [nrow,ncol]  = size(tiwt);
   
%Applying global threshold 
   if strcmp(type,'Hard') == 1
	  tiwt(:,2:ncol) = HardThresh(tiwt(:,2:ncol),thr);
   else
	  tiwt(:,2:ncol) = SoftThresh(tiwt(:,2:ncol),thr);
  end
  
%Denoised estimate
   reconstruct = IWT_TI(tiwt,h);
elseif strcmp(policy,'Universal') == 1
   
%Choosing the Universal threshold
thr = sqrt(2*log(n));

%Computing the translation-invariant table
   tiwt = FWT_TI(zsignal,lev,h);
   [nrow,ncol]  = size(tiwt);
   
%Applying global threshold 
   if strcmp(type,'Hard') == 1
	  tiwt(:,2:ncol) = HardThresh(tiwt(:,2:ncol),thr);
   else
	  tiwt(:,2:ncol) = SoftThresh(tiwt(:,2:ncol),thr);
  end
  
%Denoised estimate
   reconstruct = IWT_TI(tiwt,h);
else
   
%Choosing the 'leave-out-half' cross-validation threshold
   thr = fminsearch('cv',sqrt(2*log(n)),[],zsignal,lev,h,type);
   thr = thr/sqrt((1-log(2)/log(n)));
   
%Computing the translation-invariant table
   tiwt = FWT_TI(zsignal,lev,h);
   [nrow,ncol]  = size(tiwt);
   
%Applying global threshold 
   if strcmp(type,'Hard') == 1
	  tiwt(:,2:ncol) = HardThresh(tiwt(:,2:ncol),thr);
   else
	  tiwt(:,2:ncol) = SoftThresh(tiwt(:,2:ncol),thr);
  end
  
%Denoised estimate
   reconstruct = IWT_TI(tiwt,h);
end

%Denoised estimate of the original signal using the 
%inverse Anscombe transformation
f = (reconstruct/2).^2-3/8;

% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
