function f = reccorrectedTI(signal,lambda0,ordind,type,lev,h)

% reccorrectedTI: Smoothing Poisson data (burst type intensities) using the 
%                 translation invariant version of orthonormal wavelets and 
%                 corrected thresholds on the resulting empirical wavelet coefficients.       
% Usage
%                 f = reccorrectedTI(signal,lambda0,ordind,type,lev,h)
% Inputs
%   signal	      1-d Noisy signal, length(signal)= 2^J.
%   lambda0       Background intensity level (true value or an estimate).
%   ordind        Order indicator dictating whether skewness alone (i.e. 3) is corrected for, 
%                 or both skewness and kurtosis (i.e., 3 or 4) are corrected for. 
%                 Optional, default = 4.
%   type	         'Hard' if you want hard-thresholding; 'Soft' if you want soft-thresholding.
%		            Optional, default = hard-thresholding.
%   lev           Coarsest resolution level; lev << J=log2(length(signal)).
%                 Optional, default = 4.
%   h 		      Quadrature mirror filter for wavelet transform.
%		            Optional, default = Symmlet 8.
% Outputs
%   f		         Estimate, obtained by applying thresholding on the 
%                 wavelet coefficients.
% Notes
%                 This method does not work with lev < 4, a restriction that is required 
%                 by the TholdSolve routine called by the reccorrected function.
% References
%                 Coifman, R.R. & Donoho, D.L. (1995). Translation-invariant 
%                 de-noising. In "Wavelets and Statistics", Antoniadis, A. & 
%                 Oppenheim, G. (Eds.), Lect. Notes Statist., 103, pp. 125-150, 
%                 New York: Springer-Verlag.
%
%                 Kolaczyk, E.D. (1999). Wavelet shrinkage estimation of certain 
%                 Poisson intensity signals using corrected thresholds. "Statistica 
%                 Sinica", 9, 119-135.

if nargin < 6
   h = MakeONFilter('Symmlet',8);
end
if nargin < 5
   lev = 4;
end
if nargin < 4
	type = 'Hard';
end
if nargin <3
   ordind = 4;
end

%Initialisations
n=length(signal);
J=log2(n);

%Computing thresholds for min and max bounds.
tmax = TholdSolve(lev,J-1,n*lambda0,h,'max',ordind);
tmin = TholdSolve(lev,J-1,n*lambda0,h,'min',ordind);
if sum(isnan(tmax)) > 0 | sum(isnan(tmin)) > 0
   f = NaN*ones(1,n);
else
   
%Computing the translation-invariant table 
TIw = FWT_TI(signal,lev,h);
dnTIw = TIw;

%Applying level-dependent thresholds 
for l = 1:(J-lev),
    w  = dnTIw(:,l+1);
    posind = find(w >= 0.0);
    negind = find(w < 0.0);
    wpos = w(posind);
    wneg = w(negind);
    if strcmp(type,'Hard')==1
       wpos = wpos.*(wpos >= tmax(l));
       wneg = wneg.*(wneg <= -tmin(l));
    else
       wpos = wpos - tmax(l);
       wpos = (wpos + abs(wpos))/2;
       wneg = wneg + tmin(l);
       wneg = (wneg - abs(wneg))/2;
    end
    w(posind) = wpos;
    w(negind) = wneg;
    dnTIw(:,l+1) = w;
 end
 
%Denoised estimate
f = IWT_TI(dnTIw,h);
end

% Acknowledgement
%
% This code is based on a MatLab routine kindly provided by Eric Kolaczyk.
%
% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
