function f = recfiszTI(signal,policy,type,lev,h)

% recfiszTI:  Smoothing Poisson counts based on preprocessing the original
%             data using the translation invariant version of the Fisz 
%             transformation and applying Gaussian-based global thresholds 
%             on the resulting empirical wavelet coefficients.        
% Usage
%             f = recfiszTI(signal,policy,type,lev,h)
% Inputs
%   signal	  1-d Noisy signal, length(signal)= 2^J.
%   policy    'MinMax' if you want the minimax threshold;
%             'Universal' if you want the universal threshold;
%             'CV' if you want thresholding using the 'leave-out-half' 
%             cross-validation strategy. Optional, default = universal threshold.
%   type      'Hard' if you want hard-thresholding; 'Soft' if you want for soft-thresholding. 
%             Optional, default = hard-thresholding.	     
%   lev       Coarsest resolution level; lev << J=log2(length(signal)).
%             Optional, default = 4.
%   h 		  Quadrature mirror filter for wavelet transform.
%		        Optional, default = Symmlet 8.
% Outputs
%   f		     Estimate, obtained by using thresholding on the 
%             preprocessed wavelet coefficients and applying the
%             inverse Fisz transformation.
% Notes       
%             Other types of Gaussian-based thresholds could also be used.
%             The 'CV' with large sample sizes (i.e. n = 1024, 2048, ...)
%             is very slow and it should be avoided.
% References
%             Coifman, R.R. & Donoho, D.L. (1995). Translation-invariant 
%             de-noising. In "Wavelets and Statistics", Antoniadis, A. & 
%             Oppenheim, G. (Eds.), Lect. Notes Statist., 103, pp. 125-150, 
%             New York: Springer-Verlag.
%
%             Donoho, D.L. & Johnstone, I.M. (1994). Ideal spatial adaptation by 
%             wavelet shrinkage. Biometrika, 81, 425-455.
%
%             Fiesz, M. (1955). The limiting distribution of a function of two
%             independent random variables and its statistical applications.
%             "Collquium Mathematicum", 3, 138-146.
%
%             Fryzlewick, P. & Nason, G.P. (2001). Poisson intensity estimation
%             using wavelets and the Fisz transformation. "Technical Report", 01-10,
%             Department of Mathematics, University of Bristol, UK.
%
%             Nason, G.P. (1996). Wavelet shrinkage using cross-validation.
%             J. R. Statist. Soc. B, 58, 463-479.

if nargin < 5
	h = MakeONFilter('Symmlet',8);
end
if nargin < 4
	lev = 4;
end
if nargin < 3
	type = 'Hard';
end
if nargin < 2
   policy = 'Universal'
end

%Initialisations
n = length(signal);
J=log2(n);
m = 0;
for i=1:n
   
%Preprocessing the signal using the Fisz transform
    psignal = fisz(signal); 
    if strcmp(policy,'MinMax') == 1
              if isempty(intersect(J,[7 8 9 10]))
                 fprintf('Warning: length(signal) must be 128, 256, 512 or 1024');
                 f = NaN;
                 return;
              end
              thrhard = [2.913 3.117 3.312 3.497];
              thrsoft = [1.669 1.859 2.045 2.226];
              
%Choosing the MiniMax threshold
              if strcmp(type,'Hard') == 1
                 thr = thrhard(J-6);
              else
                 thr = thrsoft(J-6);
              end
              
%Extracting the wavelet coefficients
              wcoef = FWT_PO(psignal,lev,h);
              wthresh = wcoef;
              
%Applying global threshold
              if strcmp(type,'Hard') == 1
	             wthresh((2^lev+1):2^J) = HardThresh(wcoef((2^lev+1):2^J),thr);
              else
	             wthresh((2^lev+1):2^J) = SoftThresh(wcoef((2^lev+1):2^J),thr);
             end
             
%Denoised estimate
reconstruct = IWT_PO(wthresh,lev,h);

%Denoised estimate of the original signal using the inverse Fisz transformation
              back = fiszinv(reconstruct);
              back = [back((n-i+2):n) back(1:(n-i+1))];
              m = (1/i)*back+(1-1/i)*m;
              signal = [signal(2:n) signal(1)];
              f = m;
           elseif strcmp(policy,'Universal') == 1
              
%Extracting the wavelet coefficients
              wcoef = FWT_PO(psignal,lev,h);
              wthresh = wcoef;
              
%Applying global threshold
wthresh((2^lev+1):2^J) = VisuThresh(wcoef((2^lev+1):2^J),type);

%Denoised estimate
reconstruct = IWT_PO(wthresh,lev,h);

%Denoised estimate of the original signal using the inverse Fisz transformation
              back = fiszinv(reconstruct);
              back = [back((n-i+2):n) back(1:(n-i+1))];
              m = (1/i)*back+(1-1/i)*m;
              signal = [signal(2:n) signal(1)];
              f = m;
           else
              
%Choosing the 'leave-out-half' cross-validation threshold
              thr=fminsearch('cv',sqrt(2*log(n)),[],psignal,lev,h,type);
              thr=thr/sqrt((1-log(2)/log(n)));
              
%Extract the wavelet coefficients
              wcoef=FWT_PO(psignal,lev,h);
              wthresh = wcoef;
              
%Applying global threshold
              if strcmp(type,'Hard') == 1
                 wthresh((2^lev+1):2^J) = HardThresh(wcoef((2^lev+1):2^J),thr);
              else
                 wthresh((2^lev+1):2^J) = SoftThresh(wcoef((2^lev+1):2^J),thr);
              end
              
%denoised estimate
reconstruct = IWT_PO(wthresh,lev,h);

%denoised estimate of the original signal using the inverse Fisz transformation
              back = fiszinv(reconstruct);
              back = [back((n-i+2):n) back(1:(n-i+1))];
              m = (1/i)*back+(1-1/i)*m;
              signal = [signal(2:n) signal(1)];
              f = m;
    end
end

% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
