function f = rechaarTI(signal,lambda0,type,lev)

% rechaar:     Smoothing Poisson data (burst type intensities) using 
%              translation invariant version of Haar-based thresholds on 
%              the resulting empirical wavelet coefficients.             
% Usage
%              f = rechaarTI(signal,lambda0,type,lev)
% Inputs
%   signal	   1-d Noisy signal, length(signal)= 2^J.
%   lambda0    Background intensity level (true value or an estimate).
%   type	      'Hard' if you want hard-thresholding; 'Soft' if you want soft-thresholding.
%		         Optional, default = hard thresholding.
%   lev        Coarsest resolution level; lev << J=log2(length(signal)).
%              Optional, default = 4.
% Outputs
%   f		      Estimate, obtained by applying thresholding on the 
%              wavelet coefficients.
% References
%              Coifman, R.R. & Donoho, D.L. (1995). Translation-invariant 
%              de-noising. In "Wavelets and Statistics", Antoniadis, A. & 
%              Oppenheim, G. (Eds.), Lect. Notes Statist., 103, pp. 125-150, 
%              New York: Springer-Verlag.
%
%              Kolaczyk, E.D. (1997). Nonparametric estimation of Gamma-Ray
%              burst intensities using Haar wavelets. "The Astrophysical
%              Journal", 483, 340-349.
%
%              Kolaczyk, E.D. (1999). Wavelet shrinkage estimation of certain 
%              Poisson intensity signals using corrected thresholds. "Statistica 
%              Sinica", 9, 119-135.

if nargin < 4
   lev = 4;
end
if nargin < 3
	type = 'Hard';
end

%Initialisations
n=length(signal);
J=log2(n);

%Creating the Haar filter
h = MakeONFilter('Haar');

%Computing the translation-invariant table
TIw = FWT_TI(signal,lev,h);
dnTIw = TIw;

%Applying level-dependent thresholds 
for l = 1:(J-lev),
    w  = dnTIw(:,l+1);
    nl = 2^(J-l);
    tl = 2*log(nl);
    tl = tl+sqrt(4*((log(nl)).^2)+(8*log(nl)/nl)*n*lambda0);
    tl = tl/2^((l+2)/2);
    if strcmp(type,'Hard') == 1
       dnTIw(:,l+1) = HardThresh(w,tl);
    else 
        dnTIw(:,l+1) = SoftThresh(w,tl);
    end
 end
 
%Denoised estimete
f = IWT_TI(dnTIw,h);

% Acknowledgement
%
% This code is based on a MatLab routine kindly provided by Eric Kolaczyk.
%
% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
