function f = recmultmodulationTI(signal,lev,h)

% recmultmodulationTI:  Smoothing Poisson data using the translation invariant 
%                       version of a multiple modulation estimator based on a 
%                       partition of length [log(logn)].
% Usage
%                       f = recmultmodulationTI(signal,lev,h)
% Inputs
%   signal	            1-d Noisy signal, length(signal)= 2^J.
%   lev                 Coarsest resolution level; lev << J=log2(length(signal)).
%                       Optional, default = 4.
%   h 		            Quadrature mirror filter for wavelet transform.
%		                  Optional, default = Symmlet 8. 
% Outputs
%   f		               Estimate, obtained by applying thresholding on the 
%                       wavelet coefficients.
% References
%                       Antoniadis, A. & Sapatinas, T. (2001). Wavelet shrinkage for
%                       natural exponential families with quadratic variance functions.
%                       "Biometrika", 88, 805-820.

if nargin < 3
	h = MakeONFilter('Symmlet',8);
end
if nargin < 2
   lev = 4;
end

%Initialisations
n = length(signal);
out=zeros(1,n);

%Constructing the bin partitioning
nbin = round(log(log(n)));
nblocks = floor(n/nbin);

%Performing the cycling spin
  for i=0:(n-1),
      noistrans  = cyclespin(signal,  i);
      wcoef = FWT_PO(noistrans,lev,h);
      
%Transforming NOISTRANS to a diagonal matrix SmadreNew with diagonal NOISTRANS
%and initialising to zero the DWT2 transform of SmadreNew
      SmadreNew=diag(noistrans);
      SfigliaNew=zeros(n);
      
%2-d tensor wavelet transform (periodized, orthogonal) of SmadreNew
SFigliaNew = FTWT2_PO(SmadreNew,lev,h);

%Extracting the squared wavelet coefficients from SfigliaNew	
      sw = diag(SFigliaNew);
      sw = shapeasrow(sw);
  
      snw = max((wcoef.^2 - sqrt(n).*sw)./(wcoef.^2+eps),0).*wcoef./sqrt(2);
      newsnw = reshape(snw(1:nblocks*nbin),nbin,nblocks);
      avenewsnw = mean(newsnw);
      snw(1:nblocks*nbin)=kron(avenewsnw,ones(1,nbin));
      snw(1:(2^(lev))) = wcoef(1:(2^(lev)));

      dout  = IWT_PO(snw,lev,h);
      dout  = cyclespin(dout,  -i);
      out = out+dout;
   end
   
%Denoised estimate
f  = out/n;
   
% Copyright (c) 2000
%
% Anestis Antoniadis
% Laboratoire IMAG-LMC
% University Joseph Fourier
% BP 53, 38041 Grenoble Cedex 9
% France
%
% mailto: Anestis.Antoniadis@imag.fr
% 
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
  