function pp = MLpEstTI(dmat,qq)
	
% MLpEstTI.m:  Perform marginal maximum likelihood (MML) estimation 
%              of mixing probabilities, using TI data, via the EM algorithm.
% Usage        
%              pp = MLpEstTI(dmat,qq)
% Inputs
%   dmat       Matrix ( (J+1)xn ) of dyadic sums of data.
%   qq         Vector (Jx1) of beta shape parameters.
% Output
%   pp         Vector (Jx1) of estimated mixing probabilities.
% References
%              Kolaczyk, E.D. (1999). Bayesian Multi-Scale Models for Poisson Processes. 
%              "Journal of the American Statistical Association", 94, 920-933. 

[J,n] = size(dmat); J = J-1;

%% Create matrices of (log) probabilities to be mixed (by current estimate
%% of pp).  Given fixed values of qq, these do not ever change
%% during any of the steps of the EM algorithm.
lpmat1 = zeros(J,n);  lpmat0 = lpmat1;  lpmat = lpmat0;
for D=J:-1:1,
  nD = 2^(J-D+1);  nDo2 = nD/2;
  qval = qq(D);
  for l=0:1:2^(D-1)-1,
    ind = [l*nD+1:(l+1)*nD];
    % First half, use values in unshifted order.
    ind1h = ind(1:nDo2);
    x = dmat(D,ind);
    xl = x(1:2:nD-1); xr = x(2:2:nD);
    xx = xl+xr;
    lnck = gammaln(xx+1) - gammaln(xl+1) - gammaln(xr+1);
    lprob = lnck - xx.*log(2);
    lpmat1(D,ind1h) = lprob;
    lprob = lnck + betaln(xl+qval,xr+qval) - betaln(qval,qval);
    lpmat0(D,ind1h) = lprob;
    % Second half, use values after right-shifting.
    ind2h = ind(nDo2+1:nD);
    x = rshift(x);
    xl = x(1:2:nD-1);  xr = x(2:2:nD);
    xx = xl + xr;
    lnck = gammaln(xx+1) - gammaln(xl+1) - gammaln(xr+1);
    lprob = lnck - xx.*log(2);
    lpmat1(D,ind2h) = lprob;
    lprob = lnck + betaln(xl+qval,xr+qval) - betaln(qval,qval);
    lpmat0(D,ind2h) = lprob;
  end
end

% Begin EM algorithm for computing TI-ML estimates of mixing parameters,
% pp.  Structure is essentially that which typically arises with ML
% estimation of mixing parameters, but both mixing variables and "shift"
% variable are taken as necessary to create complete data vector.  With
% addition of the "shift" variable, formulas do not involve just simple
% products (resulting from typical likelihood structure), but sums of
% products.  Hence, components of EM equations are more complicated to
% compute.  Fortunately, redundancy in TI table allows for more efficient
% algorithm than might normally be expected.
% Iterations typically less than 10.  Tolerance is not pushed
% terribly low, as changes in pp tend to be important only on
% the order of +/- 0.10 or so.
maxit = 100;
tol = 10e-5;
convcrit = 10.0;

% Starting values may be improved with some experience on particular
% signals, but 0.5 all around seems to work well, with few iterations.
pip1 = 0.5*ones(J,1);
iter = 0;
while ( (convcrit > tol) & (iter < maxit) ),
  pi = pip1;
  
% "E-Step":  True E-step would require computing expected values
% of indicators for shift and mixing variables (products, actually),
% but these are done during the M-Step which essentially takes
% sums and products of various components of this step.  The 
% redundancy in the TI framework is exploited in that step.  For
% this step, we'll just compute the probabilities of the mixed
% distribution.
  for j=1:1:J,
    lpmat(j,:) = exp( lpmat1(j,:) + log(pi(j))  ) + ...
                 exp( lpmat0(j,:) + log(1 - pi(j))  );
  end
  lpmat = log(lpmat) ;
  
% M-Step: Values pip1 that maximize the expected log-complete
% data likelihood are the ratios of two quantities, arising from 
% the true E-Step.  These are sums, over shifts and position, of
% probabilities in pmat and pmat1.  Due to redundancies, these
% may be computed using a particular sequence of steps over and
% over again.  These steps are incorporated in the function "TISumProd.m".

% Compute numerator for each component of pip1.  There are
% increasingly fewer steps for coarser levels.
  pip1 = zeros(J,1);
  templpmat = lpmat;
  for j=1:1:J,
    partprob = zeros(1,n);
    for l=1:1:n,
      templpmat(j,l) = lpmat1(j,l) + log(pi(j));
      ratiolpmat = lpmat - templpmat;
      partprob(l) = TISumProd(ratiolpmat);
      templpmat(j,l) = lpmat(j,l);
    end
  pip1(j) = sum(ones(1,n)./partprob);
  end
  
% Bound values away from 0.
  pip1 = max(pip1, 1e-8);
  
% Test for convergence.
  p = pip1;
  
%p
%iter
  convcrit = sum( (pip1-pi).^2 )/ sum( pi.^2 );
  iter = iter + 1;
end
pp = p;

% Copyright (c) 1998.
%
% Eric Kolaczyk, 
% Department of Mathematics and Statistics
% Boston University
% 111 Cummington Avenue
% Boston, MA 02215
% USA
%
% mailto: kolaczyk@math.bu.edu 
%
% Revised: 01/09/05



